@@ -70,14 +70,34 @@ void output()
     output_debug();
     output_stype();
     if (rflag) write_section(tables);
+    if (language == PERL && perl5005flag) {
+      fprintf(code_file,
+	      "use fields qw(yylex yyerror yydebug yynerrs yyerrflag yym \n");
+      fprintf(code_file,
+	      "              yyn yyss yyssp yyvs yyvsp yystate yyval \n");
+      fprintf(code_file,
+	      "              yylval yychar yys);\n");
+    }
     write_section(header);
     if (language == C)
 	output_trailing_text();
+    if (language == PERL) {
+      fprintf(code_file, "sub new {\n");
+      if (perl5005flag) {
+	fprintf(code_file, "  my %s $p = bless [\\%FIELDS], $_[0];\n",
+		perl_package);
+      }
+      else {
+	fprintf(code_file, "  my $p = bless {}, $_[0];\n");
+      }
+    }
     write_section(body);
     output_semantic_actions();
     write_section(trailer);
-    if (language == PERL)
+    if (language == PERL) {
 	output_trailing_text();
+	fprintf(code_file, "1;\n");
+    }
     free_derives();
     free_nullable();
 }
